"""
All the ipyvuetify class override as SepalWidgets.

This file overwrite all the widgets generating by ipyvuetify to add the SepalWidget class as a parent.
It should not be modified from here as it's automatically generated.

last update: 2022-12-29
"""

import ipyvuetify as v

from sepal_ui.sepalwidgets.sepalwidget import SepalWidget


class Html(v.Html, SepalWidget):
    pass


class Alert(v.Alert, SepalWidget):
    pass


class App(v.App, SepalWidget):
    pass


class AppBar(v.AppBar, SepalWidget):
    pass


class AppBarNavIcon(v.AppBarNavIcon, SepalWidget):
    pass


class Autocomplete(v.Autocomplete, SepalWidget):
    pass


class Avatar(v.Avatar, SepalWidget):
    pass


class Badge(v.Badge, SepalWidget):
    pass


class Banner(v.Banner, SepalWidget):
    pass


class BottomNavigation(v.BottomNavigation, SepalWidget):
    pass


class BottomSheet(v.BottomSheet, SepalWidget):
    pass


class Breadcrumbs(v.Breadcrumbs, SepalWidget):
    pass


class BreadcrumbsDivider(v.BreadcrumbsDivider, SepalWidget):
    pass


class BreadcrumbsItem(v.BreadcrumbsItem, SepalWidget):
    pass


class Btn(v.Btn, SepalWidget):
    pass


class BtnToggle(v.BtnToggle, SepalWidget):
    pass


class Calendar(v.Calendar, SepalWidget):
    pass


class CalendarDaily(v.CalendarDaily, SepalWidget):
    pass


class CalendarMonthly(v.CalendarMonthly, SepalWidget):
    pass


class CalendarWeekly(v.CalendarWeekly, SepalWidget):
    pass


class Card(v.Card, SepalWidget):
    pass


class CardActions(v.CardActions, SepalWidget):
    pass


class CardSubtitle(v.CardSubtitle, SepalWidget):
    pass


class CardText(v.CardText, SepalWidget):
    pass


class CardTitle(v.CardTitle, SepalWidget):
    pass


class Carousel(v.Carousel, SepalWidget):
    pass


class CarouselItem(v.CarouselItem, SepalWidget):
    pass


class CarouselReverseTransition(v.CarouselReverseTransition, SepalWidget):
    pass


class CarouselTransition(v.CarouselTransition, SepalWidget):
    pass


class Checkbox(v.Checkbox, SepalWidget):
    pass


class Chip(v.Chip, SepalWidget):
    pass


class ChipGroup(v.ChipGroup, SepalWidget):
    pass


class Col(v.Col, SepalWidget):
    pass


class ColorPicker(v.ColorPicker, SepalWidget):
    pass


class ColorPickerCanvas(v.ColorPickerCanvas, SepalWidget):
    pass


class ColorPickerSwatches(v.ColorPickerSwatches, SepalWidget):
    pass


class Combobox(v.Combobox, SepalWidget):
    pass


class Container(v.Container, SepalWidget):
    pass


class Content(v.Content, SepalWidget):
    pass


class Counter(v.Counter, SepalWidget):
    pass


class Data(v.Data, SepalWidget):
    pass


class DataFooter(v.DataFooter, SepalWidget):
    pass


class DataIterator(v.DataIterator, SepalWidget):
    pass


class DataTable(v.DataTable, SepalWidget):
    pass


class DataTableHeader(v.DataTableHeader, SepalWidget):
    pass


class DatePicker(v.DatePicker, SepalWidget):
    pass


class DatePickerDateTable(v.DatePickerDateTable, SepalWidget):
    pass


class DatePickerHeader(v.DatePickerHeader, SepalWidget):
    pass


class DatePickerMonthTable(v.DatePickerMonthTable, SepalWidget):
    pass


class DatePickerTitle(v.DatePickerTitle, SepalWidget):
    pass


class DatePickerYears(v.DatePickerYears, SepalWidget):
    pass


class Dialog(v.Dialog, SepalWidget):
    pass


class DialogBottomTransition(v.DialogBottomTransition, SepalWidget):
    pass


class DialogTransition(v.DialogTransition, SepalWidget):
    pass


class Divider(v.Divider, SepalWidget):
    pass


class EditDialog(v.EditDialog, SepalWidget):
    pass


class ExpandTransition(v.ExpandTransition, SepalWidget):
    pass


class ExpandXTransition(v.ExpandXTransition, SepalWidget):
    pass


class ExpansionPanel(v.ExpansionPanel, SepalWidget):
    pass


class ExpansionPanelContent(v.ExpansionPanelContent, SepalWidget):
    pass


class ExpansionPanelHeader(v.ExpansionPanelHeader, SepalWidget):
    pass


class ExpansionPanels(v.ExpansionPanels, SepalWidget):
    pass


class FabTransition(v.FabTransition, SepalWidget):
    pass


class FadeTransition(v.FadeTransition, SepalWidget):
    pass


class FileInput(v.FileInput, SepalWidget):
    pass


class Flex(v.Flex, SepalWidget):
    pass


class Footer(v.Footer, SepalWidget):
    pass


class Form(v.Form, SepalWidget):
    pass


class Hover(v.Hover, SepalWidget):
    pass


class Icon(v.Icon, SepalWidget):
    pass


class Img(v.Img, SepalWidget):
    pass


class Input(v.Input, SepalWidget):
    pass


class Item(v.Item, SepalWidget):
    pass


class ItemGroup(v.ItemGroup, SepalWidget):
    pass


class Label(v.Label, SepalWidget):
    pass


class Layout(v.Layout, SepalWidget):
    pass


class Lazy(v.Lazy, SepalWidget):
    pass


class List(v.List, SepalWidget):
    pass


class ListGroup(v.ListGroup, SepalWidget):
    pass


class ListItem(v.ListItem, SepalWidget):
    pass


class ListItemAction(v.ListItemAction, SepalWidget):
    pass


class ListItemActionText(v.ListItemActionText, SepalWidget):
    pass


class ListItemAvatar(v.ListItemAvatar, SepalWidget):
    pass


class ListItemContent(v.ListItemContent, SepalWidget):
    pass


class ListItemGroup(v.ListItemGroup, SepalWidget):
    pass


class ListItemIcon(v.ListItemIcon, SepalWidget):
    pass


class ListItemSubtitle(v.ListItemSubtitle, SepalWidget):
    pass


class ListItemTitle(v.ListItemTitle, SepalWidget):
    pass


class Menu(v.Menu, SepalWidget):
    pass


class MenuTransition(v.MenuTransition, SepalWidget):
    pass


class Messages(v.Messages, SepalWidget):
    pass


class NavigationDrawer(v.NavigationDrawer, SepalWidget):
    pass


class OverflowBtn(v.OverflowBtn, SepalWidget):
    pass


class Overlay(v.Overlay, SepalWidget):
    pass


class Pagination(v.Pagination, SepalWidget):
    pass


class Parallax(v.Parallax, SepalWidget):
    pass


class Picker(v.Picker, SepalWidget):
    pass


class ProgressCircular(v.ProgressCircular, SepalWidget):
    pass


class ProgressLinear(v.ProgressLinear, SepalWidget):
    pass


class Radio(v.Radio, SepalWidget):
    pass


class RadioGroup(v.RadioGroup, SepalWidget):
    pass


class RangeSlider(v.RangeSlider, SepalWidget):
    pass


class Rating(v.Rating, SepalWidget):
    pass


class Responsive(v.Responsive, SepalWidget):
    pass


class Row(v.Row, SepalWidget):
    pass


class ScaleTransition(v.ScaleTransition, SepalWidget):
    pass


class ScrollXReverseTransition(v.ScrollXReverseTransition, SepalWidget):
    pass


class ScrollXTransition(v.ScrollXTransition, SepalWidget):
    pass


class ScrollYReverseTransition(v.ScrollYReverseTransition, SepalWidget):
    pass


class ScrollYTransition(v.ScrollYTransition, SepalWidget):
    pass


class Select(v.Select, SepalWidget):
    pass


class Sheet(v.Sheet, SepalWidget):
    pass


class SimpleCheckbox(v.SimpleCheckbox, SepalWidget):
    pass


class SimpleTable(v.SimpleTable, SepalWidget):
    pass


class SkeletonLoader(v.SkeletonLoader, SepalWidget):
    pass


class SlideGroup(v.SlideGroup, SepalWidget):
    pass


class SlideItem(v.SlideItem, SepalWidget):
    pass


class SlideXReverseTransition(v.SlideXReverseTransition, SepalWidget):
    pass


class SlideXTransition(v.SlideXTransition, SepalWidget):
    pass


class SlideYReverseTransition(v.SlideYReverseTransition, SepalWidget):
    pass


class SlideYTransition(v.SlideYTransition, SepalWidget):
    pass


class Slider(v.Slider, SepalWidget):
    pass


class Snackbar(v.Snackbar, SepalWidget):
    pass


class Spacer(v.Spacer, SepalWidget):
    pass


class Sparkline(v.Sparkline, SepalWidget):
    pass


class SpeedDial(v.SpeedDial, SepalWidget):
    pass


class Stepper(v.Stepper, SepalWidget):
    pass


class StepperContent(v.StepperContent, SepalWidget):
    pass


class StepperHeader(v.StepperHeader, SepalWidget):
    pass


class StepperItems(v.StepperItems, SepalWidget):
    pass


class StepperStep(v.StepperStep, SepalWidget):
    pass


class Subheader(v.Subheader, SepalWidget):
    pass


class Switch(v.Switch, SepalWidget):
    pass


class SystemBar(v.SystemBar, SepalWidget):
    pass


class Tab(v.Tab, SepalWidget):
    pass


class TabItem(v.TabItem, SepalWidget):
    pass


class TabReverseTransition(v.TabReverseTransition, SepalWidget):
    pass


class TabTransition(v.TabTransition, SepalWidget):
    pass


class TableOverflow(v.TableOverflow, SepalWidget):
    pass


class Tabs(v.Tabs, SepalWidget):
    pass


class TabsItems(v.TabsItems, SepalWidget):
    pass


class TabsSlider(v.TabsSlider, SepalWidget):
    pass


class Text(v.Text, SepalWidget):
    pass


class TextField(v.TextField, SepalWidget):
    pass


class Textarea(v.Textarea, SepalWidget):
    pass


class ThemeProvider(v.ThemeProvider, SepalWidget):
    pass


class TimePicker(v.TimePicker, SepalWidget):
    pass


class TimePickerClock(v.TimePickerClock, SepalWidget):
    pass


class TimePickerTitle(v.TimePickerTitle, SepalWidget):
    pass


class Timeline(v.Timeline, SepalWidget):
    pass


class TimelineItem(v.TimelineItem, SepalWidget):
    pass


class Toolbar(v.Toolbar, SepalWidget):
    pass


class ToolbarItems(v.ToolbarItems, SepalWidget):
    pass


class ToolbarTitle(v.ToolbarTitle, SepalWidget):
    pass


class Tooltip(v.Tooltip, SepalWidget):
    pass


class Treeview(v.Treeview, SepalWidget):
    pass


class TreeviewNode(v.TreeviewNode, SepalWidget):
    pass


class VirtualTable(v.VirtualTable, SepalWidget):
    pass


class Window(v.Window, SepalWidget):
    pass


class WindowItem(v.WindowItem, SepalWidget):
    pass
